# Copyright (c) 2010 gocept gmbh & co. kg
# See also LICENSE.txt

import asterisk.agi
import ConfigParser
import datetime
import gocept.support.notification
import gocept.support.rt
import gocept.support.sla
import optparse
import os.path
import sys


class EmergencyCall(object):
    """Dispatch calls to the emergency hotline.

    This code assumes that each customer can be identified uniquely with it's
    PIN (no two customers have the same PIN). A ticket is created, the pagers
    are notified and the forwarding extension is output in an Asterisk
    variable.
    """

    def __init__(self):
        parser = optparse.OptionParser()
        parser.usage = '%progname [OPTIONS] PIN'
        parser.add_option('-c', '--config', default='~/.gocept-support.cfg')
        options, args = parser.parse_args()
        try:
            self.pin = args[0]
        except IndexError:
            parser.print_help()
            parser.exit()
        self.conf = ConfigParser.ConfigParser()
        self.conf.read([os.path.expanduser(options.config)])
        self.notifications = gocept.support.notification.Notifications(
            self.conf)
        self.agi = asterisk.agi.AGI()
        self.callerid = self.agi.env.get('agi_callerid', '0')

    def lookup_period(self, sla):
        now = sla.tz.localize(datetime.datetime.now())
        return sla.period(now)

    def message(self, sla, period, forward_exten):
        subject = 'Emergency call for %s from %s (%s, %s minutes)' % (
            sla.description, self.callerid, period.description,
            period.response_time)
        body = 'Call forwarded to %s.' % forward_exten
        due = (datetime.datetime.utcnow() + datetime.timedelta(
            minutes=period.response_time)).astimezone(pytz.timezone('Europe/Berlin'))
        return (subject, body, due)

    def create_ticket(self, subject, body, due):
        rt = gocept.support.rt.RT(self.conf.get('rt', 'user'),
                                  self.conf.get('rt', 'password'),
                                  self.conf.get('rt', 'url'))
        ticket = rt.new_ticket({'Queue': self.conf.get('rt', 'queue'),
                                'Subject': subject,
                                'Text': body})
        ticket.edit({'Starts': due.strftime(rt.DATE_FORMAT)})

    def exit(self, ext):
        self.agi.goto_on_exit(extension=ext, priority='1')

    def dispatch(self):
        try:
            sla = gocept.support.sla.SLA.from_pin(self.conf, self.pin)
        except KeyError:
            print >>sys.stderr, "pin %s does not match any customer's pin" % (
                self.pin)
            self.exit('invalidpin')
            return 0
        try:
            period = self.lookup_period(sla)
        except KeyError:
            print >>sys.stderr, '%s has no support booked for %s' % (
                sla.description, datetime.datetime.now())
            self.exit('invalidtime')
            return 0
        forward_exten = self.notifications.phones[0]
        subject, body, due = self.message(sla, period, forward_exten)
        self.notifications.notify(subject, body)
        try:
            self.create_ticket(subject, body, due)
        except Exception, e:
            print >>sys.stderr, 'Ticket creation failed: %s' % e
        self.agi.set_variable('forward_exten', forward_exten)
        self.exit('forwardcall')


def emergency_call():
    c = EmergencyCall()
    exitcode = c.dispatch()
    sys.exit(exitcode)
