# Copyright (c) 2009,2010 gocept gmbh & co. kg
# See also LICENSE.txt

import ConfigParser
import datetime
import gocept.support.notification
import gocept.support.rt
import gocept.support.sla
import gocept.support.staff
import logging
import nagext
import optparse
import os
import pytz
import sys
log = logging.getLogger(__name__)


def process_notification():
    """A proxy script to deal with a notification that should be send to the
    standby support.

    This causes a ticket with a deadline to be created in the RT and
    notifications to be send out to the current supporter.

    """
    logging.basicConfig(filename='/tmp/gocept.support.nagios.log')
    parser = optparse.OptionParser()
    parser.add_option('-c', '--config', default='~/.gocept-support.cfg')
    parser.add_option('-H', '--host')
    parser.add_option('-s', '--service')
    parser.add_option('--sla')
    options, (subject,) = parser.parse_args()
    subject = subject.decode('latin1')
    conf = ConfigParser.ConfigParser()
    conf.read([os.path.expanduser(options.config)])

    message = sys.stdin.read()
    message = message.decode('latin1')

    if subject.startswith('** ACKNOW'):
        # Hard loop avoidance
        return
    # Check whether we have booked support currently
    sla = gocept.support.sla.SLA(conf.get('slas', options.sla))
    now = datetime.datetime.now(sla.tz)
    try:
        period = sla.period(now)
    except KeyError:
        # No support booked -> we ignore the notification
        return
    ticket_due = now + datetime.timedelta(minutes=period.response_time)
    customer_contacts = os.environ.get('NAGIOS__SERVICECUSTOMER_CONTACTS')
    ticket = {'Queue': conf.get('rt', 'queue'),
              'Subject': subject,
              'Text': message}
    if customer_contacts is not None:
        ticket['Cc'] = customer_contacts.split(',')
    try:
        rt = gocept.support.rt.RT(conf.get('rt', 'user'),
                                  conf.get('rt', 'password'),
                                  conf.get('rt', 'url'))
        ticket = rt.new_ticket(ticket)
        ticket.edit({'Starts': ticket_due.astimezone(
            pytz.timezone('Europe/Berlin')).strftime(rt.DATE_FORMAT)})
    except:
        log.exception('Exception while creating ticket in RT')
        ticket = {'id': 'n/a'}
    else:
        # Only acknowledge the issue within Nagios if we managed to create an
        # RT ticket.
        try:
            nagios = nagext.NagExt(conf.get('nagios', 'cmdfile'))
            if options.service is not None:
                nagios.acknowledge_svc_problem(
                    options.host, options.service, 0, 1, 1,
                'Nagios/RT integration', 'Problem reported to RT in ticket #%s' %
                    ticket['id'])
            else:
                nagios.acknowledge_host_problem(
                    options.host, 0, 1, 1,
                'Nagios/RT integration', 'Problem reported to RT in ticket #%s' %
                    ticket['id'])
        except:
            pass

    notifications = gocept.support.notification.Notifications(conf)
    notifications.notify('%smin %s:%s #%s %s' % (period.response_time,
                                                 ticket_due.hour,
                                                 ticket_due.minute,
                                                 ticket['id'],
                                                 subject), message)



def check_assigned_staff():
    parser = optparse.OptionParser()
    parser.add_option('-c', '--config', default='~/.gocept-support.cfg')
    options, args = parser.parse_args()
    conf = ConfigParser.ConfigParser()
    conf.read([os.path.expanduser(options.config)])

    staff = gocept.support.staff.Staff(conf.get('main', 'staff'))
    warning_days = []
    error_days = []
    for sla, sla_file in conf.items('slas'):
        sla = gocept.support.sla.SLA(sla_file)
        day = datetime.date.today() - datetime.timedelta(days=1)

        while day < (datetime.date.today() + datetime.timedelta(weeks=8)):
            day += datetime.timedelta(days=1)

            periods = sla.periods(day)
            if not periods:
                continue

            min_supporter = 1
            for period in periods:
                if period.response_time <= 15:
                    min_supporter = 2

            staffs = staff.assignments(day)
            if len(staffs) >= min_supporter:
                continue

            if (day - datetime.date.today()).days // 7 < 5:
                msg = error_days
            elif (day - datetime.date.today()).days // 7 < 9:
                msg = warning_days
            else:
                continue

            day_iso = day.isoformat()
            if day_iso not in msg:
                msg.append(day_iso)


    def _print(messages):
        print "Insufficient staff for: %s" % ', '.join(sorted(messages))

    if error_days:
        _print(error_days)
        sys.exit(2)

    if warning_days:
        _print(warning_days)
        sys.exit(1)

    print "OK"
    sys.exit(0)
